<?php

declare(strict_types=1);

namespace Drupal\billwerk_subscriptions;

use Drupal\billwerk_subscriptions\Exception\EnvironmentException;

/**
 * Helper class to provide environment related functionality.
 *
 * Environment can be "Production" or "Sandbox" and therefore has implications
 * on the different URLs to use. Furthermore the environments have different
 * IDs for their specific data.
 */
final class Environment {

  const API_PRODUCTION_URL = 'https://app.billwerk.com/';
  const API_PRODUCTION_SELFSERVICE_URL = 'https://selfservice.billwerk.com/';
  const API_SANDBOX_URL = 'https://sandbox.billwerk.com/';
  const API_SANDBOX_SELFSERVICE_URL = 'https://selfservice.sandbox.billwerk.com/';
  const API_OAUTH_PATH = 'oauth/token';
  const API_BASE_PATH = 'api/v1/';
  const API_CONTRACTS_RESOURCE_PATH = 'contracts';
  const API_CONTRACTCHANGES_RESOURCE_PATH = 'contractchanges';
  const API_CUSTOMERS_RESOURCE_PATH = 'customers';
  const API_CUSTOMERS_SEARCH_RESOURCE_PATH = 'customers';
  const API_DISCOUNTS_RESOURCE_PATH = 'discounts';
  const API_INVOICEDRAFTS_RESOURCE_PATH = 'invoicedrafts';
  const API_INVOICES_RESOURCE_PATH = 'invoices';
  const API_ORDERS_RESOURCE_PATH = 'orders';
  const API_PLANGROUPS_RESOURCE_PATH = 'plangroups';
  const API_PLANS_RESOURCE_PATH = 'plans';
  const API_PLANVARIANTS_RESOURCE_PATH = 'planvariants';
  const API_PRICELISTS_RESOURCE_PATH = 'pricelists';
  const API_PRODUCTINFO_RESOURCE_PATH = 'productinfo';
  const API_SUBSCRIPTIONS_RESOURCE_PATH = 'subscriptions';
  const API_WEBHOOKS_RESOURCE_PATH = 'webhooks';

  const ENVIRONMENT_PRODUCTION = 'production';

  const ENVIRONMENT_SANDBOX = 'sandbox';

  /**
   * Constructs an Environment object.
   */
  public function __construct(
        protected readonly SettingsHelper $settingsHelper
  ) {
  }


  /**
   * Returns the current environment name string.
   *
   * Environment::ENVIRONMENT_PRODUCTION
   * or
   * Environment::ENVIRONMENT_SANDBOX
   *
   * @throws \Drupal\billwerk_subscriptions\Exception\EnvironmentException
   * @return string
   */
  public function getEnvironment(): string {
    $environment = $this->config->get('environment');
    if (!in_array($environment, self::getEnvironments())) {
      throw new EnvironmentException("The configured environment \"{$environment}\" is not allowed.");
    }
    return $environment;
  }

  /**
   *
   */
  public static function getEnvironments() {
    return [
      self::ENVIRONMENT_SANDBOX,
      self::ENVIRONMENT_PRODUCTION,
    ];
  }

  /**
   * Summary of getClientId.
   *
   * @return string
   */
  public function getClientId(): string {
    return $this->isProduction() ? $this->settingsHelper->getProductionClientId() : $this->settingsHelper->getSandboxClientId();
  }

  /**
   * Summary of getClientSecret.
   *
   * @return string
   */
  public function getClientSecret(): string {
    return $this->isProduction() ? $this->settingsHelper->getProductionClientSecret() : $this->settingsHelper->getSandboxClientSecret();
  }

  /**
   * Summary of getApiUrl.
   *
   * @param string $path
   *
   * @return string
   */
  public function getApiUrl(string $path = ''): string {
    $baseUrl = $this->isProduction() ? self::API_PRODUCTION_URL : self::API_SANDBOX_URL;
    return $baseUrl . $path;
  }

  /**
   * Returns the selfservice iframe URL.
   *
   * The selfservice iframe URL is individual and secret per contract and
   * retrieved from the API using the selfServiceToken call.
   * For that reason this function requires the Billwerk API object and the
   * acting users contract ID.
   *
   * @param \Drupal\billwerk_subscriptions\Api $api
   * @param string $contractId
   *
   * @throws \Drupal\billwerk_subscriptions\Exception\EnvironmentException.
   *
   * @return string
   */
  public function getSelfserviceIframeUrl(Api $api, string $contractId): string {
    $selfServiceTokenArray = $api->getSelfserviceTokenArray($contractId);
    if (!empty($selfServiceTokenArray) && !empty($selfServiceTokenArray['Url'])) {
      return $selfServiceTokenArray['Url'];
    }
    else {
      throw new EnvironmentException('Self Service iFrame URL could not be determined for this contract.');
    }
  }

  /**
   * Summary of getApiSignupUrl.
   *
   * @param string $planVariantId
   *
   * @return string
   */
  public function getSignupIframeUrl(string $planVariantId): string {
    // @improve: This URL also allows prefilling other data for the user, see:
    // https://developer.billwerk.io/docs/gettingStarted/integratingBillwerk
    $baseUrl = $this->isProduction() ? self::API_PRODUCTION_SELFSERVICE_URL : self::API_SANDBOX_SELFSERVICE_URL;
    return $baseUrl . "/portal/signup.html#/{$planVariantId}";
  }

  /**
   * Returns true if the production environment is active.
   *
   * @return bool
   */
  public function isProduction(): bool {
    return $this->getEnvironment() === self::ENVIRONMENT_PRODUCTION;
  }

  /**
   * Returns true if the sandbox environment is active.
   *
   * @return bool
   */
  public function isSandbox(): bool {
    return $this->getEnvironment() === self::ENVIRONMENT_SANDBOX;
  }

}
