<?php

namespace Drupal\billwerk_subscriptions\Event;

use Drupal\billwerk_subscriptions\Subscriber;
use Symfony\Contracts\EventDispatcher\Event;

/**
 * High level event, fired shot before an order is created initially or as
 * up / downgrade for a contract or to add / remove a contract component.
 * See https://billwerk.readme.io/reference/orders_postorder_orderdto_post for details.
 *
 * NOTE: This is only called for backend order operations, not for frontend
 * operations using SubscriptionJS!
 */
class SubscriberOrderCreateEvent extends Event {

	/**
	 * Determines if the order should be committed directly.
	 *
	 * This does NOT work combined with $previewOrder!
	 *
	 * @var bool
	 */
	protected bool $commitOrder = TRUE;

	/**
	 * Determines if the order is only for preview.
	 *
	 * This can be useful to get pricing information or to dry-run the order.
	 * Preview orders can not be committed ($commitOrder)!
	 */
	protected bool $previewOrder = FALSE;

	/**
	 * Constructor.
	 *
	 * @param array $orderData
	 *   The prepared order data. An order array accepting the values from
	 *   https://billwerk.readme.io/reference/orders_postorder_orderdto_post
	 *   Allowed to be altered before being sent.
	 * @param mixed $subscriber
	 *   The subscriber to run the order for. Might not have a subscription /
	 *   order yet, typically!
	 */
	public function __construct(
		// This may be altered by subscribers, therefor NOT readonly:
		protected array $orderData,
		protected readonly Subscriber $subscriber,
	) {
	}

	/**
	 * Returns the order data array.
	 *
	 * @return array
	 */
	public function getOrderData(): array {
		return $this->orderData;
	}

	/**
	 * Sets the order data array.
	 *
	 * @param array $orderData
	 * @return self
	 */
	public function setOrderData(array $orderData): self {
		$this->orderData = $orderData;
		return $this;
	}

	/**
	 * Returns the Subscriber.
	 *
	 * @return \Drupal\billwerk_subscriptions\Subscriber
	 */
	public function getSubscriber(): Subscriber {
		return $this->subscriber;
	}

	/**
	 * @return bool
	 */
	public function getCommitOrder(): bool {
		return $this->commitOrder;
	}

	/**
	 * @param bool $commit
	 * @return self
	 */
	public function setCommitOrder(bool $commit): self {
		$this->commitOrder = $commit;
		return $this;
	}

	/**
	 * @return bool
	 */
	public function getPreviewOrder(): bool {
		return $this->previewOrder;
	}

	/**
	 * @param bool $preview
	 * @return self
	 */
	public function setPreviewOrder(bool $preview): self {
		$this->previewOrder = $preview;
		return $this;
	}
}
